package cn.iocoder.yudao.module.sale.controller.admin.channel;

import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseDO;
import cn.iocoder.yudao.module.depository.dto.LogisticsInfoDto;
import cn.iocoder.yudao.module.depository.service.warehouse.WarehouseService;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductAttrDO;
import cn.iocoder.yudao.module.product.service.product.ProductAttrService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.apache.commons.lang3.StringUtils;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;

import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import java.util.stream.Collectors;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.util.ExcelUtils;

import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

import cn.iocoder.yudao.module.sale.vo.channel.*;
import cn.iocoder.yudao.module.sale.dal.dataobject.channel.ChannelDO;
import cn.iocoder.yudao.module.sale.convert.channel.ChannelConvert;
import cn.iocoder.yudao.module.sale.service.channel.ChannelService;

@Api(tags = "管理后台 - 渠道管理")
@RestController
@RequestMapping("/ecw/channel")
@Validated
public class ChannelController {

    @Autowired
    private ChannelService channelService;
    @Autowired
    private WarehouseService warehouseService;
    @Autowired
    private ProductAttrService productAttrService;


    @PostMapping("/create")
    @ApiOperation("创建渠道管理")
    @PreAuthorize("@ss.hasPermission('ecw:channel:create')")
    public CommonResult<Long> createChannel(@Valid @RequestBody ChannelCreateReqVO createReqVO) {
        return success(channelService.createChannel(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("编辑渠道管理")
    @PreAuthorize("@ss.hasPermission('ecw:channel:update')")
    public CommonResult<Boolean> updateChannel(@Valid @RequestBody ChannelUpdateReqVO updateReqVO) {
        channelService.updateChannel(updateReqVO);
        return success(true);
    }


    @PutMapping("/switch/{channelId}")
    @ApiOperation("启用/禁用渠道")
    @PreAuthorize("@ss.hasPermission('ecw:channel:update')")
    public CommonResult<Boolean> updateChannelStatus(@PathVariable("channelId") Long channelId) {
        channelService.updateChannelStatus(channelId);
        return success(true);
    }

    @PutMapping("/cost/update")
    @ApiOperation("渠道费用调价管理")
    //@PreAuthorize("@ss.hasPermission('ecw:channel:update')")
    public CommonResult<Boolean> updateChannelCost(@Valid @RequestBody ChannelCostUpdateReqVO costUpdateReqVO) {
        channelService.updateChannelCost(costUpdateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除渠道管理")
    @ApiImplicitParam(name = "channelId", value = "渠道ID", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:channel:delete')")
    public CommonResult<Boolean> deleteChannel(@RequestParam("channelId") Long channelId) {
        channelService.deleteChannel(channelId);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得渠道管理")
    @ApiImplicitParam(name = "channelId", value = "渠道ID", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:channel:query')")
    public CommonResult<ChannelRespVO> getChannel(@RequestParam("channelId") Long channelId) {
        ChannelRespVO channelRespVO = channelService.getInfo(channelId);
        return success(channelRespVO);
    }


    @GetMapping("/list")
    @ApiOperation("获得渠道管理列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //@PreAuthorize("@ss.hasPermission('ecw:channel:query')")
    public CommonResult<List<ChannelRespVO>> getChannelList(@RequestParam("ids") Collection<Long> ids) {
        List<ChannelDO> list = channelService.getChannelList(ids);
        return success(ChannelConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/list-all-simple")
    @ApiOperation(value = "获得渠道精简信息列表", notes = "主要用于前端的下拉选项")
    public CommonResult<List<ChannelSimpleRespVO>> getChannelSimpleList() {
        List<ChannelDO> list = channelService.getChannelSimpleList();
        return success(ChannelConvert.INSTANCE.convertSimpleList(list));
    }

    @GetMapping("/select")
    @ApiOperation("获得可用渠道列表查询(参数可选择一个，最终以目的地国家来查询，不传代表查询所有)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "countryId", value = "目的地国家id", required = false, dataTypeClass = Long.class),
            @ApiImplicitParam(name = "warehouseId", value = "目的地仓库id", required = false, dataTypeClass = Long.class),
            @ApiImplicitParam(name = "cityId", value = "目的地城市id", required = false, dataTypeClass = Long.class),
            @ApiImplicitParam(name = "lineId", value = "线路id", required = false, dataTypeClass = Long.class)
    })
    public CommonResult<List<ChannelRespVO>> selectChannelList(Long countryId, Long cityId, Long warehouseId, Long lineId) {
        if (Objects.isNull(countryId)){
            // 国家id为空则选择其他参数传递
            if (Objects.nonNull(warehouseId)){
                WarehouseDO warehouseDO = warehouseService.getWarehouse(warehouseId);
                if (Objects.nonNull(warehouseDO)){
                    countryId = warehouseDO.getGuojia();
                }
            }
        }
        if (Objects.isNull(countryId)){
            // 国家id为空则选择其他参数传递
            if (Objects.nonNull(cityId)){
                WarehouseDO warehouseDO = warehouseService.getWarehouseByShi(cityId);
                if (Objects.nonNull(warehouseDO)){
                    countryId = warehouseDO.getGuojia();
                }
            }
        }
        if (Objects.isNull(countryId)){
            // 国家id为空则选择其他参数传递
            if (Objects.nonNull(lineId)){
                LogisticsInfoDto logisticsInfoDto = warehouseService.getStartInfoAndDestInfoByLineId(lineId);
                if (Objects.nonNull(logisticsInfoDto)){
                    countryId = logisticsInfoDto.getDestCountryId();
                }
            }
        }
        List<ChannelDO> list = channelService.list(new LambdaQueryWrapper<ChannelDO>()
                .in(Objects.nonNull(countryId), ChannelDO::getCountryId, 0,countryId)
                .eq(ChannelDO::getStatus, 1)
                .orderByAsc(ChannelDO::getSort));
        List<ChannelRespVO> resultList = ChannelConvert.INSTANCE.convertList(list);
        List<ProductAttrDO> productAttrDOList = productAttrService.getProductAttrList();
        Map<String, String> idNameMap = productAttrDOList.stream()
                .collect(Collectors.toMap(attr -> String.valueOf(attr.getId()), ProductAttrDO::getAttrName));
        for (ChannelRespVO channelRespVO : resultList) {
            channelRespVO.setAttrNameList(getAttrNameByIds(channelRespVO.getAttrId(), idNameMap));
        }
        return success(resultList);
    }

    @GetMapping("/all/select")
    @ApiOperation("获得全部的渠道信息列表(供订单详情使用，因为可能有历史订单使用的被禁用的渠道信息)")
    public CommonResult<List<ChannelRespVO>> selectAllChannelList() {
        List<ChannelDO> list = channelService.list(new LambdaQueryWrapper<ChannelDO>()
                .orderByAsc(ChannelDO::getSort));
        List<ChannelRespVO> resultList = ChannelConvert.INSTANCE.convertList(list);
        List<ProductAttrDO> productAttrDOList = productAttrService.getProductAttrList();
        Map<String, String> idNameMap = productAttrDOList.stream()
                .collect(Collectors.toMap(attr -> String.valueOf(attr.getId()), ProductAttrDO::getAttrName));
        for (ChannelRespVO channelRespVO : resultList) {
            channelRespVO.setAttrNameList(getAttrNameByIds(channelRespVO.getAttrId(), idNameMap));
        }
        return success(resultList);
    }

    private List<String> getAttrNameByIds(String attrIds, Map<String, String> idNameMap) {
        if(StringUtils.isNotBlank(attrIds)) {
            String[] attrIdArray = attrIds.split(",");
            List<String> nameList = new ArrayList<>();
            for (String attrId : attrIdArray) {
                //map一定包含数据，否则是数据错误
                nameList.add(idNameMap.get(attrId));
            }
            return nameList;
        }
        return null;
    }

    @GetMapping("/page")
    @ApiOperation("获得渠道管理分页")
    //@PreAuthorize("@ss.hasPermission('ecw:channel:query')")
    public CommonResult<PageResult<ChannelRespVO>> getChannelPage(@Valid ChannelPageReqVO pageVO) {
        PageResult<ChannelRespVO> pageResult = channelService.channelPage(pageVO);
        return success(pageResult);
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出渠道管理 Excel")
    //@PreAuthorize("@ss.hasPermission('ecw:channel:export')")
    @OperateLog(type = EXPORT)
    public void exportChannelExcel(@Valid ChannelExportReqVO exportReqVO,
              HttpServletResponse response) throws IOException {
        List<ChannelDO> list = channelService.getChannelList(exportReqVO);
        // 导出 Excel
        List<ChannelExcelVO> datas = ChannelConvert.INSTANCE.convertList02(list);
        ExcelUtils.write(response, I18nMessage.getLang() == 0 ? "渠道管理.xls" : "channel-manage.xls", "数据", ChannelExcelVO.class, datas);
    }

}
